# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['kanidm', 'tests']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.8.1,<4.0.0', 'pydantic>=1.9.2,<2.0.0', 'toml>=0.10.2,<0.11.0']

setup_kwargs = {
    'name': 'kanidm',
    'version': '0.0.2',
    'description': 'Kanidm client library',
    'long_description': "# kanidm\n\nA Python module for interacting with Kanidm.\n\nCurrently in very very very early beta, please [log an issue](https://github.com/kanidm/kanidm/issues/new/choose) for feature requests and bugs.\n\n## Installation\n\n```shell\npython -m pip install kanidm\n```\n\n## Documentation\n\nDocumentation can be generated by [cloning the repository](https://github.com/kanidm/kanidm) and running `make docs/pykanidm/build`. The documentation will appear in `./pykanidm/site`. You'll need make and the [poetry](https://pypi.org/project/poetry/) package installed.\n\n\n## Changelog\n\n| Version | Date | Notes |\n| ---     | ---  | ---   |\n| 0.0.1   | 2022-08-16 | Initial release |\n| 0.0.2   | 2022-08-16 | Updated license, including test code in package |\n",
    'author': 'James Hodgkinson',
    'author_email': 'james@terminaloutcomes.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://kanidm.com/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
